package com.supervision.websocket;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.lock.annotation.Lock4j;
import com.supervision.constant.UserTokenConstant;
import com.supervision.exception.BusinessException;
import com.supervision.websocket.dto.KickDTO;
import com.supervision.websocket.dto.NoResourceDTO;
import com.supervision.websocket.dto.WebSocketLoginDTO;
import lombok.Data;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;

import javax.websocket.*;

import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.io.IOException;

@Slf4j
@Component
@RequiredArgsConstructor
public class UserResourceCheck {

    @Value("${human.resourceMaxNumber}")
    private String resourceNumber;

    private final RedisTemplate<String, String> redisTemplate;

    @Lock4j(name = "achieveDiagnoseResource")
    public boolean achieveDiagnoseResource() {
        long humanMaxNumber = Long.parseLong(resourceNumber);
        long currentSize = redisTemplate.opsForHash().size(UserTokenConstant.USER_WEBSOCKET_CACHE);
        // 如果小于数字人最大连接数,则可以连接
        return currentSize < humanMaxNumber;
    }

    @Lock4j(name = "achieveDiagnoseResourceAndOpenConnection")
    public void achieveDiagnoseResourceAndOpenConnection(String uuid, String userId, Session session) {
        // 如果小于数字人最大连接数,则可以连接
        if (!achieveDiagnoseResource()) {
            try {
                session.getBasicRemote().sendText(JSONUtil.toJsonStr(new NoResourceDTO()));
            } catch (Exception e) {
                log.error("发送消息失败", e);
            }
            throw new BusinessException("暂时没有资源,建立连接失败");
        }
        redisTemplate.opsForHash().put(UserTokenConstant.USER_WEBSOCKET_CACHE, uuid, JSONUtil.toJsonStr(new WebSocketLoginDTO(uuid, userId, session.getId())));
    }

    /**
     * websocket保活接口,每5秒钟发送一次消息
     */

    // 实现一个方法用于踢下线用户,走的是Redis的消息队列
    public void kickUser(String uuid) {
        log.info("尝试踢uuid:{}下线", uuid);
        Session session = WebSocketSessionPool.SESSION_POOL.get(uuid);
        // 只有不是忽略剔除的sessionId才可以踢下线
        if (ObjectUtil.isNotEmpty(session)) {
            try {
                session.getBasicRemote().sendText(JSONUtil.toJsonStr(new KickDTO()));
                session.close(new CloseReason(CloseReason.CloseCodes.NORMAL_CLOSURE, "用户被踢下线"));
                WebSocketSessionPool.SESSION_POOL.remove(uuid);
                // 然后从redis中移除掉
                redisTemplate.opsForHash().delete(UserTokenConstant.USER_WEBSOCKET_CACHE, uuid);
                log.info("踢UUID:{},sessionId:{} 下线成功", uuid, session.getId());
                return;
            } catch (IOException e) {
                log.error("用户:{}的websocket连接异常", e.getMessage());
                // TODO 如果用户连接异常,怎么办

            }
        }
        log.info("踢UUID:{}下线,未找到用户,踢下线失败", uuid);
    }


}
