package com.supervision.websocket;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import com.supervision.websocket.UserResourceCheck;
import com.supervision.websocket.WebSocketSessionPool;
import com.supervision.websocket.dto.KeepaliveDTO;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.websocket.Session;
import java.io.IOException;
import java.util.Map;

@Component
@Slf4j
@RequiredArgsConstructor
public class WebsocketKeepaliveTask {
    @Scheduled(fixedDelay = 5 * 1000L)
    public void keepalive() {
        for (Map.Entry<String, Session> entries : WebSocketSessionPool.SESSION_POOL.entrySet()) {
            String userId = entries.getKey();
            Session session = entries.getValue();
            if (ObjectUtil.isNotEmpty(session)) {
                try {
                    session.getBasicRemote().sendText(JSONUtil.toJsonStr(new KeepaliveDTO()));
                    log.info("用户:{}的websocket保活成功", userId);
                } catch (IOException e) {
                    log.error("用户:{}的websocket连接异常", userId, e);
                }
            }
        }
    }

}
